@@ -16,72 +16,14 @@
 
 """Searching in versioned file repositories."""
 
-from __future__ import absolute_import
-
 import itertools
 
-from .. import (
-    debug,
-    revision,
-    trace,
-    )
-
-from ..graph import (
-    DictParentsProvider,
-    Graph,
-    invert_parent_map,
-    )
-from ..sixish import (
-    viewvalues,
-    )
+from .. import debug, revision, trace
+from ..graph import DictParentsProvider, Graph, invert_parent_map
+from ..repository import AbstractSearchResult
 
 
-class AbstractSearchResult(object):
-    """The result of a search, describing a set of keys.
-
-    Search results are typically used as the 'fetch_spec' parameter when
-    fetching revisions.
-
-    :seealso: AbstractSearch
-    """
-
-    def get_recipe(self):
-        """Return a recipe that can be used to replay this search.
-
-        The recipe allows reconstruction of the same results at a later date.
-
-        :return: A tuple of `(search_kind_str, *details)`.  The details vary by
-            kind of search result.
-        """
-        raise NotImplementedError(self.get_recipe)
-
-    def get_network_struct(self):
-        """Return a tuple that can be transmitted via the HPSS protocol."""
-        raise NotImplementedError(self.get_network_struct)
-
-    def get_keys(self):
-        """Return the keys found in this search.
-
-        :return: A set of keys.
-        """
-        raise NotImplementedError(self.get_keys)
-
-    def is_empty(self):
-        """Return false if the search lists 1 or more revisions."""
-        raise NotImplementedError(self.is_empty)
-
-    def refine(self, seen, referenced):
-        """Create a new search by refining this search.
-
-        :param seen: Revisions that have been satisfied.
-        :param referenced: Revision references observed while satisfying some
-            of this search.
-        :return: A search result.
-        """
-        raise NotImplementedError(self.refine)
-
-
-class AbstractSearch(object):
+class AbstractSearch:
     """A search that can be executed, producing a search result.
 
     :seealso: AbstractSearchResult
@@ -116,21 +58,20 @@ class SearchResult(AbstractSearchResult):
             a SearchResult from a smart server, in which case the keys list is
             not necessarily immediately available.
         """
-        self._recipe = ('search', start_keys, exclude_keys, key_count)
+        self._recipe = ("search", start_keys, exclude_keys, key_count)
         self._keys = frozenset(keys)
 
     def __repr__(self):
         kind, start_keys, exclude_keys, key_count = self._recipe
         if len(start_keys) > 5:
-            start_keys_repr = repr(list(start_keys)[:5])[:-1] + ', ...]'
+            start_keys_repr = repr(list(start_keys)[:5])[:-1] + ", ...]"
         else:
             start_keys_repr = repr(start_keys)
         if len(exclude_keys) > 5:
-            exclude_keys_repr = repr(list(exclude_keys)[:5])[:-1] + ', ...]'
+            exclude_keys_repr = repr(list(exclude_keys)[:5])[:-1] + ", ...]"
         else:
             exclude_keys_repr = repr(exclude_keys)
-        return '<%s %s:(%s, %s, %d)>' % (self.__class__.__name__,
-                                         kind, start_keys_repr, exclude_keys_repr, key_count)
+        return f"<{self.__class__.__name__} {kind}:({start_keys_repr}, {exclude_keys_repr}, {key_count})>"
 
     def get_recipe(self):
         """Return a recipe that can be used to replay this search.
@@ -156,11 +97,13 @@ class SearchResult(AbstractSearchResult):
         return self._recipe
 
     def get_network_struct(self):
-        start_keys = b' '.join(self._recipe[1])
-        stop_keys = b' '.join(self._recipe[2])
-        count = str(self._recipe[3]).encode('ascii')
-        return (self._recipe[0].encode('ascii'),
-                b'\n'.join((start_keys, stop_keys, count)))
+        start_keys = b" ".join(self._recipe[1])
+        stop_keys = b" ".join(self._recipe[2])
+        count = str(self._recipe[3]).encode("ascii")
+        return (
+            self._recipe[0].encode("ascii"),
+            b"\n".join((start_keys, stop_keys, count)),
+        )
 
     def get_keys(self):
         """Return the keys found in this search.
@@ -219,11 +162,12 @@ class PendingAncestryResult(AbstractSearchResult):
     def __repr__(self):
         if len(self.heads) > 5:
             heads_repr = repr(list(self.heads)[:5])[:-1]
-            heads_repr += ', <%d more>...]' % (len(self.heads) - 5,)
+            heads_repr += f", <{len(self.heads) - 5} more>...]"
         else:
             heads_repr = repr(self.heads)
-        return '<%s heads:%s repo:%r>' % (
-            self.__class__.__name__, heads_repr, self.repo)
+        return "<{} heads:{} repo:{!r}>".format(
+            self.__class__.__name__, heads_repr, self.repo
+        )
 
     def get_recipe(self):
         """Return a recipe that can be used to replay this search.
@@ -236,10 +180,10 @@ class PendingAncestryResult(AbstractSearchResult):
             To recreate this result, create a PendingAncestryResult with the
             start_keys_set.
         """
-        return ('proxy-search', self.heads, set(), -1)
+        return ("proxy-search", self.heads, set(), -1)
 
     def get_network_struct(self):
-        parts = [b'ancestry-of']
+        parts = [b"ancestry-of"]
         parts.extend(self.heads)
         return parts
 
@@ -253,8 +197,11 @@ class PendingAncestryResult(AbstractSearchResult):
 
     def _get_keys(self, graph):
         NULL_REVISION = revision.NULL_REVISION
-        keys = [key for (key, parents) in graph.iter_ancestry(self.heads)
-                if key != NULL_REVISION and parents is not None]
+        keys = [
+            key
+            for (key, parents) in graph.iter_ancestry(self.heads)
+            if key != NULL_REVISION and parents is not None
+        ]
         return keys
 
     def is_empty(self):
@@ -289,21 +236,23 @@ class EverythingResult(AbstractSearchResult):
         self._repo = repo
 
     def __repr__(self):
-        return '%s(%r)' % (self.__class__.__name__, self._repo)
+        return "{}({!r})".format(self.__class__.__name__, self._repo)
 
     def get_recipe(self):
         raise NotImplementedError(self.get_recipe)
 
     def get_network_struct(self):
-        return (b'everything',)
+        return (b"everything",)
 
     def get_keys(self):
-        if 'evil' in debug.debug_flags:
+        if "evil" in debug.debug_flags:
             from . import remote
+
             if isinstance(self._repo, remote.RemoteRepository):
                 # warn developers (not users) not to do this
                 trace.mutter_callsite(
-                    2, "EverythingResult(RemoteRepository).get_keys() is slow.")
+                    2, "EverythingResult(RemoteRepository).get_keys() is slow."
+                )
         return self._repo.all_revision_ids()
 
     def is_empty(self):
@@ -329,14 +278,22 @@ class EverythingNotInOther(AbstractSearch):
 
     def execute(self):
         return self.to_repo.search_missing_revision_ids(
-            self.from_repo, find_ghosts=self.find_ghosts)
+            self.from_repo, find_ghosts=self.find_ghosts
+        )
 
 
 class NotInOtherForRevs(AbstractSearch):
     """Find all revisions missing in one repo for a some specific heads."""
 
-    def __init__(self, to_repo, from_repo, required_ids, if_present_ids=None,
-                 find_ghosts=False, limit=None):
+    def __init__(
+        self,
+        to_repo,
+        from_repo,
+        required_ids,
+        if_present_ids=None,
+        find_ghosts=False,
+        limit=None,
+    ):
         """Constructor.
 
         :param required_ids: revision IDs of heads that must be found, or else
@@ -357,25 +314,34 @@ class NotInOtherForRevs(AbstractSearch):
 
     def __repr__(self):
         if len(self.required_ids) > 5:
-            reqd_revs_repr = repr(list(self.required_ids)[:5])[:-1] + ', ...]'
+            reqd_revs_repr = repr(list(self.required_ids)[:5])[:-1] + ", ...]"
         else:
             reqd_revs_repr = repr(self.required_ids)
         if self.if_present_ids and len(self.if_present_ids) > 5:
-            ifp_revs_repr = repr(list(self.if_present_ids)[:5])[:-1] + ', ...]'
+            ifp_revs_repr = repr(list(self.if_present_ids)[:5])[:-1] + ", ...]"
         else:
             ifp_revs_repr = repr(self.if_present_ids)
 
-        return ("<%s from:%r to:%r find_ghosts:%r req'd:%r if-present:%r"
-                "limit:%r>") % (
-            self.__class__.__name__, self.from_repo, self.to_repo,
-            self.find_ghosts, reqd_revs_repr, ifp_revs_repr,
-            self.limit)
+        return (
+            "<{} from:{!r} to:{!r} find_ghosts:{!r} req'd:{!r} if-present:{!r}limit:{!r}>"
+        ).format(
+            self.__class__.__name__,
+            self.from_repo,
+            self.to_repo,
+            self.find_ghosts,
+            reqd_revs_repr,
+            ifp_revs_repr,
+            self.limit,
+        )
 
     def execute(self):
         return self.to_repo.search_missing_revision_ids(
-            self.from_repo, revision_ids=self.required_ids,
-            if_present_ids=self.if_present_ids, find_ghosts=self.find_ghosts,
-            limit=self.limit)
+            self.from_repo,
+            revision_ids=self.required_ids,
+            if_present_ids=self.if_present_ids,
+            find_ghosts=self.find_ghosts,
+            limit=self.limit,
+        )
 
 
 def search_result_from_parent_map(parent_map, missing_keys):
@@ -386,14 +352,16 @@ def search_result_from_parent_map(parent_map, missing_keys):
     # start_set is all the keys in the cache
     start_set = set(parent_map)
     # result set is all the references to keys in the cache
-    result_parents = set(itertools.chain.from_iterable(viewvalues(parent_map)))
+    result_parents = set(itertools.chain.from_iterable(parent_map.values()))
     stop_keys = result_parents.difference(start_set)
     # We don't need to send ghosts back to the server as a position to
     # stop either.
     stop_keys.difference_update(missing_keys)
     key_count = len(parent_map)
-    if (revision.NULL_REVISION in result_parents
-            and revision.NULL_REVISION in missing_keys):
+    if (
+        revision.NULL_REVISION in result_parents
+        and revision.NULL_REVISION in missing_keys
+    ):
         # If we pruned NULL_REVISION from the stop_keys because it's also
         # in our cache of "missing" keys we need to increment our key count
         # by 1, because the reconsitituted SearchResult on the server will
@@ -424,14 +392,14 @@ def _run_search(parent_map, heads, exclude_keys):
             next_revs = next(s)
         except StopIteration:
             break
-        for parents in viewvalues(s._current_parents):
+        for parents in s._current_parents.values():
             f_heads = heads.intersection(parents)
             if f_heads:
                 found_heads.update(f_heads)
         stop_keys = exclude_keys.intersection(next_revs)
         if stop_keys:
             s.stop_searching_any(stop_keys)
-    for parents in viewvalues(s._current_parents):
+    for parents in s._current_parents.values():
         f_heads = heads.intersection(parents)
         if f_heads:
             found_heads.update(f_heads)
@@ -472,8 +440,7 @@ def _find_possible_heads(parent_map, tip_keys, depth):
     return heads
 
 
-def limited_search_result_from_parent_map(parent_map, missing_keys, tip_keys,
-                                          depth):
+def limited_search_result_from_parent_map(parent_map, missing_keys, tip_keys, depth):
     """Transform a parent_map that is searching 'tip_keys' into an
     approximate SearchResult.
 
