@@ -22,43 +22,47 @@
 # importing this module is fairly slow because it has to load several
 # ElementTree bits
 
+__all__ = [
+    "Element",
+    "ElementTree",
+    "SubElement",
+    "XMLSerializer",
+    "escape_invalid_chars",
+    "fromstring",
+    "fromstringlist",
+    "get_utf8_or_ascii",
+    "serialize_inventory_flat",
+    "tostring",
+    "tostringlist",
+    "unpack_inventory_entry",
+    "unpack_inventory_flat",
+]
+
 import re
+from typing import Union
+from xml.etree.ElementTree import (
+    Element,
+    ElementTree,
+    ParseError,
+    SubElement,
+    fromstring,
+    fromstringlist,
+    tostring,
+    tostringlist,
+)
 
-try:
-    import xml.etree.cElementTree as elementtree
-    from xml.etree.ElementTree import ParseError
-except ImportError:
-    # Fall back to pure python implementation if C extension is unavailable
-    import xml.etree.ElementTree as elementtree
-    try:
-        from xml.etree.ElementTree import ParseError
-    except ImportError:
-        from xml.parsers.expat import ExpatError as ParseError
-
-(ElementTree, SubElement, Element, fromstringlist, tostringlist, tostring,
- fromstring) = (
-    elementtree.ElementTree, elementtree.SubElement, elementtree.Element,
-    elementtree.fromstringlist, elementtree.tostringlist, elementtree.tostring,
-    elementtree.fromstring)
-
-
-from .. import (
-    errors,
-    lazy_regex,
-    )
-from . import (
-    inventory,
-    serializer,
-    )
+from .. import lazy_regex
+from . import inventory, serializer
 
 
 class XMLSerializer(serializer.Serializer):
-    """Abstract XML object serialize/deserialize"""
+    """Abstract XML object serialize/deserialize."""
 
     squashes_xml_invalid_characters = True
 
-    def read_inventory_from_lines(self, lines, revision_id=None,
-                                  entry_cache=None, return_from_cache=False):
+    def read_inventory_from_lines(
+        self, lines, revision_id=None, entry_cache=None, return_from_cache=False
+    ):
         """Read xml_string into an inventory object.
 
         :param chunks: The xml to read.
@@ -78,30 +82,32 @@ class XMLSerializer(serializer.Serializer):
             make some operations significantly faster.
         """
         try:
-            return self._unpack_inventory(fromstringlist(lines), revision_id,
-                                          entry_cache=entry_cache,
-                                          return_from_cache=return_from_cache)
+            return self._unpack_inventory(
+                fromstringlist(lines),
+                revision_id,
+                entry_cache=entry_cache,
+                return_from_cache=return_from_cache,
+            )
         except ParseError as e:
             raise serializer.UnexpectedInventoryFormat(str(e))
 
     def read_inventory(self, f, revision_id=None):
         try:
             try:
-                return self._unpack_inventory(self._read_element(f),
-                                              revision_id=None)
+                return self._unpack_inventory(self._read_element(f), revision_id=None)
             finally:
                 f.close()
         except ParseError as e:
             raise serializer.UnexpectedInventoryFormat(str(e))
 
     def write_revision_to_string(self, rev):
-        return b''.join(self.write_revision_to_lines(rev))
+        return b"".join(self.write_revision_to_lines(rev))
 
     def read_revision(self, f):
         return self._unpack_revision(self._read_element(f))
 
     def read_revision_from_string(self, xml_string):
-        return self._unpack_revision(fromstring(xml_string))
+        return self._unpack_revision(fromstring(xml_string))  # noqa: S314
 
     def _read_element(self, f):
         return ElementTree().parse(f)
@@ -119,10 +125,11 @@ def escape_invalid_chars(message):
     # represented in well-formed XML; escape characters that
     # aren't listed in the XML specification
     # (http://www.w3.org/TR/REC-xml/#NT-Char).
-    return re.subn(u'[^\x09\x0A\x0D\u0020-\uD7FF\uE000-\uFFFD]+',
-                   lambda match: match.group(0).encode(
-                       'unicode_escape').decode('ascii'),
-                   message)
+    return re.subn(
+        "[^\x09\x0a\x0d\u0020-\ud7ff\ue000-\ufffd]+",
+        lambda match: match.group(0).encode("unicode_escape").decode("ascii"),
+        message,
+    )
 
 
 def get_utf8_or_ascii(a_str):
@@ -141,26 +148,26 @@ def get_utf8_or_ascii(a_str):
     # not meant as a generic function for all cases. Because it is possible for
     # an 8-bit string to not be ascii or valid utf8.
     if a_str.__class__ is str:
-        return a_str.encode('utf-8')
+        return a_str.encode("utf-8")
     else:
         return a_str
 
 
-_utf8_re = lazy_regex.lazy_compile(b'[&<>\'\"]|[\x80-\xff]+')
-_unicode_re = lazy_regex.lazy_compile(u'[&<>\'\"\u0080-\uffff]')
+_utf8_re = lazy_regex.lazy_compile(b"[&<>'\"]|[\x80-\xff]+")
+_unicode_re = lazy_regex.lazy_compile("[&<>'\"\u0080-\uffff]")
 
 
 _xml_escape_map = {
-    "&": '&amp;',
+    "&": "&amp;",
     "'": "&apos;",  # FIXME: overkill
-    "\"": "&quot;",
+    '"': "&quot;",
     "<": "&lt;",
     ">": "&gt;",
-    }
+}
 
 
 def _unicode_escape_replace(match, _map=_xml_escape_map):
-    """Replace a string of non-ascii, non XML safe characters with their escape
+    """Replace a string of non-ascii, non XML safe characters with their escape.
 
     This will escape both Standard XML escapes, like <>"', etc.
     As well as escaping non ascii characters, because ElementTree did.
@@ -175,7 +182,7 @@ def _unicode_escape_replace(match, _map=_xml_escape_map):
     try:
         return _map[match.group()]
     except KeyError:
-        return "&#%d;" % ord(match.group())
+        return f"&#{ord(match.group())};"
 
 
 def _utf8_escape_replace(match, _map=_xml_escape_map):
@@ -187,17 +194,18 @@ def _utf8_escape_replace(match, _map=_xml_escape_map):
     decode back into Unicode, and then use the XML escape code.
     """
     try:
-        return _map[match.group().decode('ascii', 'replace')].encode()
+        return _map[match.group().decode("ascii", "replace")].encode()
     except KeyError:
-        return b''.join(b'&#%d;' % ord(uni_chr)
-                        for uni_chr in match.group().decode('utf8'))
+        return b"".join(
+            b"&#%d;" % ord(uni_chr) for uni_chr in match.group().decode("utf8")
+        )
 
 
-_to_escaped_map = {}
+_to_escaped_map: dict[Union[bytes, str], str] = {}
 
 
 def encode_and_escape(unicode_or_utf8_str, _map=_to_escaped_map):
-    """Encode the string into utf8, and escape invalid XML characters"""
+    """Encode the string into utf8, and escape invalid XML characters."""
     # We frequently get entities we have not seen before, so it is better
     # to check if None, rather than try/KeyError
     text = _map.get(unicode_or_utf8_str)
@@ -211,25 +219,25 @@ def encode_and_escape(unicode_or_utf8_str, _map=_to_escaped_map):
             # better than entity escapes, but cElementTree seems to do just
             # fine either way)
             text = _unicode_re.sub(
-                _unicode_escape_replace, unicode_or_utf8_str).encode()
+                _unicode_escape_replace, unicode_or_utf8_str
+            ).encode()
         else:
             # Plain strings are considered to already be in utf-8 so we do a
             # slightly different method for escaping.
-            text = _utf8_re.sub(_utf8_escape_replace,
-                                unicode_or_utf8_str)
+            text = _utf8_re.sub(_utf8_escape_replace, unicode_or_utf8_str)
         _map[unicode_or_utf8_str] = text
     return text
 
 
 def _clear_cache():
-    """Clean out the unicode => escaped map"""
+    """Clean out the unicode => escaped map."""
     _to_escaped_map.clear()
 
 
 def unpack_inventory_entry(elt, entry_cache=None, return_from_cache=False):
     elt_get = elt.get
-    file_id = elt_get('file_id')
-    revision = elt_get('revision')
+    file_id = elt_get("file_id")
+    revision = elt_get("revision")
     # Check and see if we have already unpacked this exact entry
     # Some timings for "repo.revision_trees(last_100_revs)"
     #               bzr     mysql
@@ -271,50 +279,45 @@ def unpack_inventory_entry(elt, entry_cache=None, return_from_cache=False):
         else:
             # Only copying directory entries drops us 2.85s => 2.35s
             if return_from_cache:
-                if cached_ie.kind == 'directory':
+                if cached_ie.kind == "directory":
                     return cached_ie.copy()
                 return cached_ie
             return cached_ie.copy()
 
     kind = elt.tag
     if not inventory.InventoryEntry.versionable_kind(kind):
-        raise AssertionError('unsupported entry kind %s' % kind)
+        raise AssertionError("unsupported entry kind {}".format(kind))
 
     file_id = get_utf8_or_ascii(file_id)
     if revision is not None:
         revision = get_utf8_or_ascii(revision)
-    parent_id = elt_get('parent_id')
+    parent_id = elt_get("parent_id")
     if parent_id is not None:
         parent_id = get_utf8_or_ascii(parent_id)
 
-    if kind == 'directory':
-        ie = inventory.InventoryDirectory(file_id,
-                                          elt_get('name'),
-                                          parent_id)
-    elif kind == 'file':
-        ie = inventory.InventoryFile(file_id,
-                                     elt_get('name'),
-                                     parent_id)
-        ie.text_sha1 = elt_get('text_sha1')
+    if kind == "directory":
+        ie = inventory.InventoryDirectory(file_id, elt_get("name"), parent_id)
+    elif kind == "file":
+        ie = inventory.InventoryFile(file_id, elt_get("name"), parent_id)
+        ie.text_sha1 = elt_get("text_sha1")
         if ie.text_sha1 is not None:
-            ie.text_sha1 = ie.text_sha1.encode('ascii')
-        if elt_get('executable') == 'yes':
+            ie.text_sha1 = ie.text_sha1.encode("ascii")
+        if elt_get("executable") == "yes":
             ie.executable = True
-        v = elt_get('text_size')
+        v = elt_get("text_size")
         ie.text_size = v and int(v)
-    elif kind == 'symlink':
-        ie = inventory.InventoryLink(file_id,
-                                     elt_get('name'),
-                                     parent_id)
-        ie.symlink_target = elt_get('symlink_target')
-    elif kind == 'tree-reference':
-        file_id = get_utf8_or_ascii(elt.attrib['file_id'])
-        name = elt.attrib['name']
-        parent_id = get_utf8_or_ascii(elt.attrib['parent_id'])
-        revision = get_utf8_or_ascii(elt.get('revision'))
-        reference_revision = get_utf8_or_ascii(elt.get('reference_revision'))
-        ie = inventory.TreeReference(file_id, name, parent_id, revision,
-                                     reference_revision)
+    elif kind == "symlink":
+        ie = inventory.InventoryLink(file_id, elt_get("name"), parent_id)
+        ie.symlink_target = elt_get("symlink_target")
+    elif kind == "tree-reference":
+        file_id = get_utf8_or_ascii(elt.attrib["file_id"])
+        name = elt.attrib["name"]
+        parent_id = get_utf8_or_ascii(elt.attrib["parent_id"])
+        revision = get_utf8_or_ascii(elt.get("revision"))
+        reference_revision = get_utf8_or_ascii(elt.get("reference_revision"))
+        ie = inventory.TreeReference(
+            file_id, name, parent_id, revision, reference_revision
+        )
     else:
         raise serializer.UnsupportedInventoryKind(kind)
     ie.revision = revision
@@ -328,8 +331,9 @@ def unpack_inventory_entry(elt, entry_cache=None, return_from_cache=False):
     return ie
 
 
-def unpack_inventory_flat(elt, format_num, unpack_entry,
-                          entry_cache=None, return_from_cache=False):
+def unpack_inventory_flat(
+    elt, format_num, unpack_entry, entry_cache=None, return_from_cache=False
+):
     """Unpack a flat XML inventory.
 
     :param elt: XML element for the inventory
@@ -339,15 +343,16 @@ def unpack_inventory_flat(elt, format_num, unpack_entry,
     :raise UnexpectedInventoryFormat: When unexpected elements or data is
         encountered
     """
-    if elt.tag != 'inventory':
-        raise serializer.UnexpectedInventoryFormat('Root tag is %r' % elt.tag)
-    format = elt.get('format')
-    if ((format is None and format_num is not None) or
-            format.encode() != format_num):
-        raise serializer.UnexpectedInventoryFormat('Invalid format version %r' % format)
-    revision_id = elt.get('revision_id')
+    if elt.tag != "inventory":
+        raise serializer.UnexpectedInventoryFormat("Root tag is {!r}".format(elt.tag))
+    format = elt.get("format")
+    if (format is None and format_num is not None) or format.encode() != format_num:
+        raise serializer.UnexpectedInventoryFormat(
+            "Invalid format version {!r}".format(format)
+        )
+    revision_id = elt.get("revision_id")
     if revision_id is not None:
-        revision_id = revision_id.encode('utf-8')
+        revision_id = revision_id.encode("utf-8")
     inv = inventory.Inventory(root_id=None, revision_id=revision_id)
     for e in elt:
         ie = unpack_entry(e, entry_cache, return_from_cache)
@@ -366,71 +371,109 @@ def serialize_inventory_flat(inv, append, root_id, supported_kinds, working):
     entries = inv.iter_entries()
     # Skip the root
     root_path, root_ie = next(entries)
-    for path, ie in entries:
+    for _path, ie in entries:
         if ie.parent_id != root_id:
-            parent_str = b''.join(
-                [b' parent_id="', encode_and_escape(ie.parent_id), b'"'])
+            parent_str = b"".join(
+                [b' parent_id="', encode_and_escape(ie.parent_id), b'"']
+            )
         else:
-            parent_str = b''
-        if ie.kind == 'file':
+            parent_str = b""
+        if ie.kind == "file":
             if ie.executable:
                 executable = b' executable="yes"'
             else:
-                executable = b''
+                executable = b""
             if not working:
-                append(b'<file%s file_id="%s" name="%s"%s revision="%s" '
-                       b'text_sha1="%s" text_size="%d" />\n' % (
-                           executable, encode_and_escape(ie.file_id),
-                           encode_and_escape(ie.name), parent_str,
-                           encode_and_escape(ie.revision), ie.text_sha1,
-                           ie.text_size))
+                append(
+                    b'<file%s file_id="%s" name="%s"%s revision="%s" '
+                    b'text_sha1="%s" text_size="%d" />\n'
+                    % (
+                        executable,
+                        encode_and_escape(ie.file_id),
+                        encode_and_escape(ie.name),
+                        parent_str,
+                        encode_and_escape(ie.revision),
+                        ie.text_sha1,
+                        ie.text_size,
+                    )
+                )
             else:
-                append(b'<file%s file_id="%s" name="%s"%s />\n' % (
-                    executable, encode_and_escape(ie.file_id),
-                    encode_and_escape(ie.name), parent_str))
-        elif ie.kind == 'directory':
+                append(
+                    b'<file%s file_id="%s" name="%s"%s />\n'
+                    % (
+                        executable,
+                        encode_and_escape(ie.file_id),
+                        encode_and_escape(ie.name),
+                        parent_str,
+                    )
+                )
+        elif ie.kind == "directory":
             if not working:
-                append(b'<directory file_id="%s" name="%s"%s revision="%s" '
-                       b'/>\n' % (
-                           encode_and_escape(ie.file_id),
-                           encode_and_escape(ie.name),
-                           parent_str,
-                           encode_and_escape(ie.revision)))
+                append(
+                    b'<directory file_id="%s" name="%s"%s revision="%s" '
+                    b"/>\n"
+                    % (
+                        encode_and_escape(ie.file_id),
+                        encode_and_escape(ie.name),
+                        parent_str,
+                        encode_and_escape(ie.revision),
+                    )
+                )
             else:
-                append(b'<directory file_id="%s" name="%s"%s />\n' % (
-                    encode_and_escape(ie.file_id),
-                    encode_and_escape(ie.name),
-                    parent_str))
-        elif ie.kind == 'symlink':
+                append(
+                    b'<directory file_id="%s" name="%s"%s />\n'
+                    % (
+                        encode_and_escape(ie.file_id),
+                        encode_and_escape(ie.name),
+                        parent_str,
+                    )
+                )
+        elif ie.kind == "symlink":
             if not working:
-                append(b'<symlink file_id="%s" name="%s"%s revision="%s" '
-                       b'symlink_target="%s" />\n' % (
-                           encode_and_escape(ie.file_id),
-                           encode_and_escape(ie.name),
-                           parent_str,
-                           encode_and_escape(ie.revision),
-                           encode_and_escape(ie.symlink_target)))
+                append(
+                    b'<symlink file_id="%s" name="%s"%s revision="%s" '
+                    b'symlink_target="%s" />\n'
+                    % (
+                        encode_and_escape(ie.file_id),
+                        encode_and_escape(ie.name),
+                        parent_str,
+                        encode_and_escape(ie.revision),
+                        encode_and_escape(ie.symlink_target),
+                    )
+                )
             else:
-                append(b'<symlink file_id="%s" name="%s"%s />\n' % (
-                    encode_and_escape(ie.file_id),
-                    encode_and_escape(ie.name),
-                    parent_str))
-        elif ie.kind == 'tree-reference':
+                append(
+                    b'<symlink file_id="%s" name="%s"%s />\n'
+                    % (
+                        encode_and_escape(ie.file_id),
+                        encode_and_escape(ie.name),
+                        parent_str,
+                    )
+                )
+        elif ie.kind == "tree-reference":
             if ie.kind not in supported_kinds:
                 raise serializer.UnsupportedInventoryKind(ie.kind)
             if not working:
-                append(b'<tree-reference file_id="%s" name="%s"%s '
-                       b'revision="%s" reference_revision="%s" />\n' % (
-                           encode_and_escape(ie.file_id),
-                           encode_and_escape(ie.name),
-                           parent_str,
-                           encode_and_escape(ie.revision),
-                           encode_and_escape(ie.reference_revision)))
+                append(
+                    b'<tree-reference file_id="%s" name="%s"%s '
+                    b'revision="%s" reference_revision="%s" />\n'
+                    % (
+                        encode_and_escape(ie.file_id),
+                        encode_and_escape(ie.name),
+                        parent_str,
+                        encode_and_escape(ie.revision),
+                        encode_and_escape(ie.reference_revision),
+                    )
+                )
             else:
-                append(b'<tree-reference file_id="%s" name="%s"%s />\n' % (
-                    encode_and_escape(ie.file_id),
-                    encode_and_escape(ie.name),
-                    parent_str))
+                append(
+                    b'<tree-reference file_id="%s" name="%s"%s />\n'
+                    % (
+                        encode_and_escape(ie.file_id),
+                        encode_and_escape(ie.name),
+                        parent_str,
+                    )
+                )
         else:
             raise serializer.UnsupportedInventoryKind(ie.kind)
-    append(b'</inventory>\n')
+    append(b"</inventory>\n")
