@@ -16,75 +16,120 @@
 
 """A Git repository implementation that uses a Bazaar transport."""
 
+import os
+import posixpath
+import sys
 from io import BytesIO
 
-import os
-import sys
-import posixpath
-
-from dulwich.errors import (
-    NoIndexPresent,
-    )
-from dulwich.file import (
-    GitFile,
-    FileLocked,
-    )
-from dulwich.objects import (
-    ShaFile,
-    )
+from dulwich.errors import NoIndexPresent
+from dulwich.file import FileLocked, _GitFile
 from dulwich.object_store import (
-    PackBasedObjectStore,
+    PACK_MODE,
     PACKDIR,
+    PackBasedObjectStore,
     read_packs_file,
-    )
+)
+from dulwich.objects import ShaFile
 from dulwich.pack import (
-    MemoryPackIndex,
+    Pack,
     PackData,
     PackIndexer,
-    Pack,
     PackStreamCopier,
+    extend_pack,
     iter_sha1,
     load_pack_index_file,
-    write_pack_objects,
-    write_pack_index_v2,
-    write_pack_header,
-    compute_file_sha,
-    write_pack_object,
-    )
+    write_pack_index,
+)
+from dulwich.refs import SymrefLoop
 from dulwich.repo import (
-    BaseRepo,
-    InfoRefsContainer,
-    RefsContainer,
     BASE_DIRECTORIES,
     COMMONDIR,
     CONTROLDIR,
     INDEX_FILENAME,
     OBJECTDIR,
     SYMREF,
+    BaseRepo,
+    InfoRefsContainer,
+    RefsContainer,
     check_ref_format,
-    read_packed_refs_with_peeled,
     read_packed_refs,
+    read_packed_refs_with_peeled,
     write_packed_refs,
-    )
+)
 
-from .. import (
-    osutils,
-    transport as _mod_transport,
-    urlutils,
-    )
+from .. import osutils, urlutils
+from .. import transport as _mod_transport
 from ..errors import (
     AlreadyControlDirError,
-    FileExists,
     LockBroken,
     LockContention,
     NotLocalUrl,
-    NoSuchFile,
     ReadError,
     TransportNotPossible,
-    )
-
+)
 from ..lock import LogicalLockResult
 from ..trace import warning
+from ..transport import FileExists, NoSuchFile
+
+
+class _RemoteGitFile:
+    def __init__(self, transport, filename, mode, bufsize, mask):
+        self.transport = transport
+        self.filename = filename
+        self.mode = mode
+        self.bufsize = bufsize
+        self.mask = mask
+        import tempfile
+
+        self._file = tempfile.SpooledTemporaryFile(max_size=1024 * 1024)
+        self._closed = False
+        for method in _GitFile.PROXY_METHODS:
+            if hasattr(self._file, method):
+                setattr(self, method, getattr(self._file, method))
+
+    def abort(self):
+        self._file.close()
+        self._closed = True
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        if exc_type is not None:
+            self.abort()
+        else:
+            self.close()
+
+    def __getattr__(self, name):
+        """Proxy property calls to the underlying file."""
+        if name in _GitFile.PROXY_PROPERTIES:
+            return getattr(self._file, name)
+        raise AttributeError(name)
+
+    def close(self):
+        if self._closed:
+            return
+        self._file.flush()
+        self._file.seek(0)
+        self.transport.put_file(self.filename, self._file)
+        self._file.close()
+        self._closed = True
+
+
+def TransportGitFile(transport, filename, mode="rb", bufsize=-1, mask=0o644):
+    if "a" in mode:
+        raise OSError("append mode not supported for Git files")
+    if "+" in mode:
+        raise OSError("read/write mode not supported for Git files")
+    if "b" not in mode:
+        raise OSError("text mode not supported for Git files")
+    if "w" in mode:
+        try:
+            return _GitFile(transport.local_abspath(filename), mode, bufsize, mask)
+        except NotLocalUrl:
+            return _RemoteGitFile(transport, filename, mode, bufsize, mask)
+    else:
+        return transport.get(filename)
 
 
 class TransportRefsContainer(RefsContainer):
@@ -99,7 +144,7 @@ class TransportRefsContainer(RefsContainer):
         self._peeled_refs = None
 
     def __repr__(self):
-        return "%s(%r)" % (self.__class__.__name__, self.transport)
+        return "{}({!r})".format(self.__class__.__name__, self.transport)
 
     def _ensure_dir_exists(self, path):
         self.transport.clone(posixpath.dirname(path)).create_prefix()
@@ -127,8 +172,7 @@ class TransportRefsContainer(RefsContainer):
         else:
             keys.add(b"HEAD")
         try:
-            iter_files = list(self.transport.clone(
-                "refs").iter_files_recursive())
+            iter_files = list(self.transport.clone("refs").iter_files_recursive())
             for filename in iter_files:
                 unquoted_filename = urlutils.unquote_to_bytes(filename)
                 refname = osutils.pathjoin(b"refs", unquoted_filename)
@@ -159,8 +203,7 @@ class TransportRefsContainer(RefsContainer):
                 return {}
             try:
                 first_line = next(iter(f)).rstrip()
-                if (first_line.startswith(b"# pack-refs") and b" peeled" in
-                        first_line):
+                if first_line.startswith(b"# pack-refs") and b" peeled" in first_line:
                     for sha, name, peeled in read_packed_refs_with_peeled(f):
                         self._packed_refs[name] = sha
                         if peeled:
@@ -202,7 +245,7 @@ class TransportRefsContainer(RefsContainer):
             exist.
         :raises IOError: if any other error occurs
         """
-        if name == b'HEAD':
+        if name == b"HEAD":
             transport = self.worktree_transport
         else:
             transport = self.transport
@@ -248,13 +291,12 @@ class TransportRefsContainer(RefsContainer):
         """
         self._check_refname(name)
         self._check_refname(other)
-        if name != b'HEAD':
+        if name != b"HEAD":
             transport = self.transport
             self._ensure_dir_exists(urlutils.quote_from_bytes(name))
         else:
             transport = self.worktree_transport
-        transport.put_bytes(urlutils.quote_from_bytes(
-            name), SYMREF + other + b'\n')
+        transport.put_bytes(urlutils.quote_from_bytes(name), SYMREF + other + b"\n")
 
     def set_if_equals(self, name, old_ref, new_ref):
         """Set a refname to new_ref only if it currently equals old_ref.
@@ -272,15 +314,14 @@ class TransportRefsContainer(RefsContainer):
         try:
             realnames, _ = self.follow(name)
             realname = realnames[-1]
-        except (KeyError, IndexError):
+        except (KeyError, IndexError, SymrefLoop):
             realname = name
-        if realname == b'HEAD':
+        if realname == b"HEAD":
             transport = self.worktree_transport
         else:
             transport = self.transport
             self._ensure_dir_exists(urlutils.quote_from_bytes(realname))
-        transport.put_bytes(urlutils.quote_from_bytes(
-            realname), new_ref + b"\n")
+        transport.put_bytes(urlutils.quote_from_bytes(realname), new_ref + b"\n")
         return True
 
     def add_if_new(self, name, ref):
@@ -301,7 +342,7 @@ class TransportRefsContainer(RefsContainer):
         except (KeyError, IndexError):
             realname = name
         self._check_refname(realname)
-        if realname == b'HEAD':
+        if realname == b"HEAD":
             transport = self.worktree_transport
         else:
             transport = self.transport
@@ -322,7 +363,7 @@ class TransportRefsContainer(RefsContainer):
         """
         self._check_refname(name)
         # may only be packed
-        if name == b'HEAD':
+        if name == b"HEAD":
             transport = self.worktree_transport
         else:
             transport = self.transport
@@ -358,20 +399,20 @@ class TransportRefsContainer(RefsContainer):
         self._ensure_dir_exists(urlutils.quote_from_bytes(name))
         lockname = urlutils.quote_from_bytes(name + b".lock")
         try:
-            local_path = transport.local_abspath(
-                urlutils.quote_from_bytes(name))
+            transport.local_abspath(urlutils.quote_from_bytes(name))
         except NotLocalUrl:
             # This is racy, but what can we do?
             if transport.has(lockname):
                 raise LockContention(name)
-            transport.put_bytes(lockname, b'Locked by brz-git')
+            transport.put_bytes(lockname, b"Locked by brz-git")
             return LogicalLockResult(lambda: transport.delete(lockname))
         else:
             try:
-                gf = GitFile(local_path, 'wb')
+                gf = TransportGitFile(transport, urlutils.quote_from_bytes(name), "wb")
             except FileLocked as e:
                 raise LockContention(name, e)
             else:
+
                 def unlock():
                     try:
                         transport.delete(lockname)
@@ -380,6 +421,7 @@ class TransportRefsContainer(RefsContainer):
                     # GitFile.abort doesn't care if the lock has already
                     # disappeared
                     gf.abort()
+
                 return LogicalLockResult(unlock)
 
 
@@ -396,11 +438,10 @@ def read_gitfile(f):
     cs = f.read()
     if not cs.startswith(b"gitdir: "):
         raise ValueError("Expected file to start with 'gitdir: '")
-    return cs[len(b"gitdir: "):].rstrip(b"\n")
+    return cs[len(b"gitdir: ") :].rstrip(b"\n")
 
 
 class TransportRepo(BaseRepo):
-
     def __init__(self, transport, bare, refs_text=None):
         self.transport = transport
         self.bare = bare
@@ -411,45 +452,50 @@ class TransportRepo(BaseRepo):
             if self.bare:
                 self._controltransport = self.transport
             else:
-                self._controltransport = self.transport.clone('.git')
+                self._controltransport = self.transport.clone(".git")
         else:
             self._controltransport = self.transport.clone(
-                urlutils.quote_from_bytes(path))
+                urlutils.quote_from_bytes(path)
+            )
         commondir = self.get_named_file(COMMONDIR)
         if commondir is not None:
             with commondir:
                 commondir = os.path.join(
                     self.controldir(),
-                    commondir.read().rstrip(b"\r\n").decode(
-                        sys.getfilesystemencoding()))
-                self._commontransport = \
-                    _mod_transport.get_transport_from_path(commondir)
+                    commondir.read()
+                    .rstrip(b"\r\n")
+                    .decode(sys.getfilesystemencoding()),
+                )
+                self._commontransport = _mod_transport.get_transport_from_path(
+                    commondir
+                )
         else:
             self._commontransport = self._controltransport
         config = self.get_config()
         object_store = TransportObjectStore.from_config(
-            self._commontransport.clone(OBJECTDIR),
-            config)
+            self._commontransport.clone(OBJECTDIR), config
+        )
         if refs_text is not None:
             refs_container = InfoRefsContainer(BytesIO(refs_text))
             try:
-                head = TransportRefsContainer(
-                    self._commontransport).read_loose_ref(b"HEAD")
+                head = TransportRefsContainer(self._commontransport).read_loose_ref(
+                    b"HEAD"
+                )
             except KeyError:
                 pass
             else:
                 refs_container._refs[b"HEAD"] = head
         else:
             refs_container = TransportRefsContainer(
-                self._commontransport, self._controltransport)
-        super(TransportRepo, self).__init__(object_store,
-                                            refs_container)
+                self._commontransport, self._controltransport
+            )
+        super().__init__(object_store, refs_container)
 
     def controldir(self):
-        return self._controltransport.local_abspath('.')
+        return self._controltransport.local_abspath(".")
 
     def commondir(self):
-        return self._commontransport.local_abspath('.')
+        return self._commontransport.local_abspath(".")
 
     def close(self):
         """Close any files opened by this repository."""
@@ -457,14 +503,21 @@ class TransportRepo(BaseRepo):
 
     @property
     def path(self):
-        return self.transport.local_abspath('.')
+        return self.transport.local_abspath(".")
 
     def _determine_file_mode(self):
         # Be consistent with bzr
-        if sys.platform == 'win32':
+        if sys.platform == "win32":
             return False
         return True
 
+    def _determine_symlinks(self):
+        try:
+            return osutils.supports_symlinks(self.path)
+        except NotLocalUrl:
+            # TODO(jelmer): Query the transport
+            return sys.platform != "win32"
+
     def get_named_file(self, path):
         """Get a file from the control dir with a specific name.
 
@@ -476,7 +529,7 @@ class TransportRepo(BaseRepo):
         :return: An open file object, or None if the file does not exist.
         """
         try:
-            return self._controltransport.get(path.lstrip('/'))
+            return self._controltransport.get(path.lstrip("/"))
         except NoSuchFile:
             return None
 
@@ -490,6 +543,7 @@ class TransportRepo(BaseRepo):
     def open_index(self):
         """Open the index for this repository."""
         from dulwich.index import Index
+
         if not self.has_index():
             raise NoIndexPresent()
         return Index(self.index_path())
@@ -502,14 +556,16 @@ class TransportRepo(BaseRepo):
 
     def get_config(self):
         from dulwich.config import ConfigFile
+
         try:
-            with self._controltransport.get('config') as f:
+            with self._controltransport.get("config") as f:
                 return ConfigFile.from_file(f)
         except NoSuchFile:
             return ConfigFile()
 
     def get_config_stack(self):
         from dulwich.config import StackedConfig
+
         backends = []
         p = self.get_config()
         if p is not None:
@@ -521,7 +577,7 @@ class TransportRepo(BaseRepo):
         return StackedConfig(backends, writable=writable)
 
     def __repr__(self):
-        return "<%s for %r>" % (self.__class__.__name__, self.transport)
+        return "<{} for {!r}>".format(self.__class__.__name__, self.transport)
 
     @classmethod
     def init(cls, transport, bare=False):
@@ -552,13 +608,14 @@ class TransportRepo(BaseRepo):
 class TransportObjectStore(PackBasedObjectStore):
     """Git-style object store that exists on disk."""
 
-    def __init__(self, transport,
-                 loose_compression_level=-1, pack_compression_level=-1):
+    def __init__(
+        self, transport, loose_compression_level=-1, pack_compression_level=-1
+    ):
         """Open an object store.
 
         :param transport: Transport to open data from
         """
-        super(TransportObjectStore, self).__init__()
+        super().__init__()
         self.pack_compression_level = pack_compression_level
         self.loose_compression_level = loose_compression_level
         self.transport = transport
@@ -568,18 +625,21 @@ class TransportObjectStore(PackBasedObjectStore):
     @classmethod
     def from_config(cls, path, config):
         try:
-            default_compression_level = int(config.get(
-                (b'core', ), b'compression').decode())
+            default_compression_level = int(
+                config.get((b"core",), b"compression").decode()
+            )
         except KeyError:
             default_compression_level = -1
         try:
-            loose_compression_level = int(config.get(
-                (b'core', ), b'looseCompression').decode())
+            loose_compression_level = int(
+                config.get((b"core",), b"looseCompression").decode()
+            )
         except KeyError:
             loose_compression_level = default_compression_level
         try:
-            pack_compression_level = int(config.get(
-                (b'core', ), 'packCompression').decode())
+            pack_compression_level = int(
+                config.get((b"core",), "packCompression").decode()
+            )
         except KeyError:
             pack_compression_level = default_compression_level
         return cls(path, loose_compression_level, pack_compression_level)
@@ -590,7 +650,7 @@ class TransportObjectStore(PackBasedObjectStore):
         return self.transport == other.transport
 
     def __repr__(self):
-        return "%s(%r)" % (self.__class__.__name__, self.transport)
+        return "{}({!r})".format(self.__class__.__name__, self.transport)
 
     @property
     def alternates(self):
@@ -627,29 +687,17 @@ class TransportObjectStore(PackBasedObjectStore):
                 try:
                     size = self.pack_transport.stat(pack_name).st_size
                 except TransportNotPossible:
-                    try:
-                        f = self.pack_transport.get(pack_name)
-                    except NoSuchFile:
-                        warning('Unable to read pack file %s',
-                                self.pack_transport.abspath(pack_name))
-                        continue
-                    # TODO(jelmer): Don't read entire file into memory?
-                    f = BytesIO(f.read())
-                    pd = PackData(pack_name, f)
-                else:
-                    pd = PackData(
-                        pack_name, self.pack_transport.get(pack_name),
-                        size=size)
+                    size = None
+                pd = PackData(pack_name, self.pack_transport.get(pack_name), size=size)
                 idxname = basename + ".idx"
-                idx = load_pack_index_file(
-                    idxname, self.pack_transport.get(idxname))
+                idx = load_pack_index_file(idxname, self.pack_transport.get(idxname))
                 pack = Pack.from_objects(pd, idx)
                 pack._basename = basename
                 self._pack_cache[basename] = pack
                 new_packs.append(pack)
         # Remove disappeared pack files
-        for f in set(self._pack_cache) - pack_files:
-            self._pack_cache.pop(f).close()
+        for n in set(self._pack_cache) - pack_files:
+            self._pack_cache.pop(n).close()
         return new_packs
 
     def _pack_names(self):
@@ -665,15 +713,17 @@ class TransportObjectStore(PackBasedObjectStore):
                         pack_files.append(os.path.splitext(name)[0])
         except TransportNotPossible:
             try:
-                f = self.transport.get('info/packs')
+                f = self.transport.get("info/packs")
             except NoSuchFile:
-                warning('No info/packs on remote host;'
-                        'run \'git update-server-info\' on remote.')
+                warning(
+                    "No info/packs on remote host;"
+                    "run 'git update-server-info' on remote."
+                )
             else:
                 with f:
                     pack_files = [
-                        os.path.splitext(name)[0]
-                        for name in read_packs_file(f)]
+                        os.path.splitext(name)[0] for name in read_packs_file(f)
+                    ]
         except NoSuchFile:
             pass
         return pack_files
@@ -687,7 +737,7 @@ class TransportObjectStore(PackBasedObjectStore):
             pass
 
     def _iter_loose_objects(self):
-        for base in self.transport.list_dir('.'):
+        for base in self.transport.list_dir("."):
             if len(base) != 2:
                 continue
             for rest in self.transport.list_dir(base):
@@ -700,6 +750,13 @@ class TransportObjectStore(PackBasedObjectStore):
         path = osutils.joinpath(self._split_loose_object(sha))
         self.transport.delete(urlutils.quote_from_bytes(path))
 
+    def delete_loose_object(self, sha):
+        """Delete a loose object.
+
+        This method is called by dulwich's pack_loose_objects method.
+        """
+        self._remove_loose_object(sha)
+
     def _get_loose_object(self, sha):
         path = osutils.joinpath(self._split_loose_object(sha))
         try:
@@ -725,83 +782,16 @@ class TransportObjectStore(PackBasedObjectStore):
         # the compression_level parameter.
         if self.loose_compression_level not in (-1, None):
             raw_string = obj.as_legacy_object(
-                compression_level=self.loose_compression_level)
+                compression_level=self.loose_compression_level
+            )
         else:
             raw_string = obj.as_legacy_object()
         self.transport.put_bytes(path, raw_string)
 
-    def move_in_pack(self, f):
-        """Move a specific file containing a pack into the pack directory.
-
-        :note: The file should be on the same file system as the
-            packs directory.
-
-        :param path: Path to the pack file.
-        """
-        f.seek(0)
-        p = PackData("", f, len(f.getvalue()))
-        entries = p.sorted_entries()
-        basename = "pack-%s" % iter_sha1(entry[0]
-                                         for entry in entries).decode('ascii')
-        p._filename = basename + ".pack"
-        f.seek(0)
-        self.pack_transport.put_file(basename + ".pack", f)
-        with self.pack_transport.open_write_stream(basename + ".idx") as idxfile:
-            write_pack_index_v2(idxfile, entries, p.get_stored_checksum())
-        idxfile = self.pack_transport.get(basename + ".idx")
-        idx = load_pack_index_file(basename + ".idx", idxfile)
-        final_pack = Pack.from_objects(p, idx)
-        final_pack._basename = basename
-        self._add_cached_pack(basename, final_pack)
-        return final_pack
-
-    def move_in_thin_pack(self, f):
-        """Move a specific file containing a pack into the pack directory.
-
-        :note: The file should be on the same file system as the
-            packs directory.
-
-        :param path: Path to the pack file.
-        """
-        f.seek(0)
-        p = Pack('', resolve_ext_ref=self.get_raw)
-        p._data = PackData.from_file(f, len(f.getvalue()))
-        p._data.pack = p
-        p._idx_load = lambda: MemoryPackIndex(
-            p.data.sorted_entries(), p.data.get_stored_checksum())
-
-        pack_sha = p.index.objects_sha1()
-
-        with self.pack_transport.open_write_stream(
-                "pack-%s.pack" % pack_sha.decode('ascii')) as datafile:
-            entries, data_sum = write_pack_objects(datafile, p.pack_tuples())
-        entries = sorted([(k, v[0], v[1]) for (k, v) in entries.items()])
-        with self.pack_transport.open_write_stream(
-                "pack-%s.idx" % pack_sha.decode('ascii')) as idxfile:
-            write_pack_index_v2(idxfile, entries, data_sum)
-
-    def add_pack(self):
-        """Add a new pack to this object store.
-
-        :return: Fileobject to write to and a commit function to
-            call when the pack is finished.
-        """
-        f = BytesIO()
-
-        def commit():
-            if len(f.getvalue()) > 0:
-                return self.move_in_pack(f)
-            else:
-                return None
-
-        def abort():
-            return None
-        return f, commit, abort
-
     @classmethod
     def init(cls, transport):
         try:
-            transport.mkdir('info')
+            transport.mkdir("info")
         except FileExists:
             pass
         try:
@@ -810,13 +800,7 @@ class TransportObjectStore(PackBasedObjectStore):
             pass
         return cls(transport)
 
-    def _get_pack_basepath(self, entries):
-        suffix = iter_sha1(entry[0] for entry in entries)
-        # TODO: Handle self.pack_dir being bytes
-        suffix = suffix.decode('ascii')
-        return self.pack_transport.local_abspath("pack-" + suffix)
-
-    def _complete_thin_pack(self, f, path, copier, indexer):
+    def _complete_pack(self, f, path, num_objects, indexer, progress=None):
         """Move a specific file containing a pack into the pack directory.
 
         Note: The file should be on the same file system as the
@@ -825,64 +809,83 @@ class TransportObjectStore(PackBasedObjectStore):
         Args:
           f: Open file object for the pack.
           path: Path to the pack file.
-          copier: A PackStreamCopier to use for writing pack data.
           indexer: A PackIndexer for indexing the pack.
         """
-        entries = list(indexer)
+        entries = []
+        for i, entry in enumerate(indexer):
+            if progress is not None:
+                progress(f"generating index: {i}/{num_objects}\r".encode("ascii"))
+            entries.append(entry)
 
-        # Update the header with the new number of objects.
-        f.seek(0)
-        write_pack_header(f, len(entries) + len(indexer.ext_refs()))
-
-        # Must flush before reading (http://bugs.python.org/issue3207)
+        pack_sha, extra_entries = extend_pack(
+            f,
+            indexer.ext_refs(),
+            get_raw=self.get_raw,
+            compression_level=self.pack_compression_level,
+            progress=progress,
+        )
         f.flush()
+        try:
+            fileno = f.fileno()
+        except AttributeError:
+            pass
+        else:
+            os.fsync(fileno)
 
-        # Rescan the rest of the pack, computing the SHA with the new header.
-        new_sha = compute_file_sha(f, end_ofs=-20)
-
-        # Must reposition before writing (http://bugs.python.org/issue3207)
-        f.seek(0, os.SEEK_CUR)
-
-        # Complete the pack.
-        for ext_sha in indexer.ext_refs():
-            type_num, data = self.get_raw(ext_sha)
-            offset = f.tell()
-            crc32 = write_pack_object(
-                f, type_num, data, sha=new_sha,
-                compression_level=self.pack_compression_level)
-            entries.append((ext_sha, offset, crc32))
-        pack_sha = new_sha.digest()
-        f.write(pack_sha)
-        f.close()
+        entries.extend(extra_entries)
 
         # Move the pack in.
         entries.sort()
-        pack_base_name = self._get_pack_basepath(entries)
-        target_pack = pack_base_name + '.pack'
-        if sys.platform == 'win32':
+        pack_base_name = "pack-" + iter_sha1(entry[0] for entry in entries).decode(
+            "ascii"
+        )
+
+        for pack in self.packs:
+            if osutils.basename(pack._basename) == pack_base_name:
+                f.close()
+                return pack
+
+        target_pack_name = pack_base_name + ".pack"
+        target_pack_index = pack_base_name + ".idx"
+        if sys.platform == "win32":
             # Windows might have the target pack file lingering. Attempt
             # removal, silently passing if the target does not exist.
-            try:
-                os.remove(target_pack)
-            except FileNotFoundError:
-                pass
-        os.rename(path, target_pack)
+            with suppress(NoSuchFile):
+                self.transport.remove(target_pack_name)
+
+        if path:
+            f.close()
+            self.pack_transport.ensure_base()
+            os.rename(
+                path,
+                self.transport.local_abspath(
+                    osutils.pathjoin(PACKDIR, target_pack_name)
+                ),
+            )
+        else:
+            f.seek(0)
+            self.pack_transport.put_file(target_pack_name, f, mode=PACK_MODE)
 
         # Write the index.
-        index_file = GitFile(pack_base_name + '.idx', 'wb')
-        try:
-            write_pack_index_v2(index_file, entries, pack_sha)
-            index_file.close()
-        finally:
-            index_file.abort()
+        with TransportGitFile(
+            self.pack_transport, target_pack_index, "wb", mask=PACK_MODE
+        ) as index_file:
+            write_pack_index(index_file, entries, pack_sha)
 
         # Add the pack to the store and return it.
-        final_pack = Pack(pack_base_name)
+        final_pack = Pack.from_objects(
+            PackData(target_pack_name, self.pack_transport.get(target_pack_name)),
+            load_pack_index_file(
+                target_pack_index, self.pack_transport.get(target_pack_index)
+            ),
+        )
+        final_pack._basename = pack_base_name
+
         final_pack.check_length_and_checksum()
         self._add_cached_pack(pack_base_name, final_pack)
         return final_pack
 
-    def add_thin_pack(self, read_all, read_some):
+    def add_thin_pack(self, read_all, read_some, progress=None):
         """Add a new thin pack to this object store.
 
         Thin packs are packs that contain deltas with parents that exist
@@ -898,10 +901,62 @@ class TransportObjectStore(PackBasedObjectStore):
             objects/pack directory.
         """
         import tempfile
-        fd, path = tempfile.mkstemp(dir=self.pack_transport.local_abspath('.'), prefix='tmp_pack_')
-        with os.fdopen(fd, 'w+b') as f:
+
+        try:
+            dir = self.transport.local_abspath(".")
+        except NotLocalUrl:
+            f = tempfile.SpooledTemporaryFile(prefix="tmp_pack_")
+            path = None
+        else:
+            f = tempfile.NamedTemporaryFile(dir=dir, prefix="tmp_pack_", delete=False)
+            path = f.name
+
+        try:
             indexer = PackIndexer(f, resolve_ext_ref=self.get_raw)
-            copier = PackStreamCopier(read_all, read_some, f,
-                                      delta_iter=indexer)
-            copier.verify()
-            return self._complete_thin_pack(f, path, copier, indexer)
+            copier = PackStreamCopier(read_all, read_some, f, delta_iter=indexer)
+            copier.verify(progress=progress)
+            if f.name:
+                os.chmod(f.name, PACK_MODE)
+            return self._complete_pack(f, path, len(copier), indexer, progress=progress)
+        except BaseException:
+            f.close()
+            if path:
+                os.remove(path)
+            raise
+
+    def add_pack(self):
+        """Add a new pack to this object store.
+
+        Returns: Fileobject to write to, a commit function to
+            call when the pack is finished and an abort
+            function.
+        """
+        import tempfile
+
+        try:
+            dir = self.transport.local_abspath(".")
+        except NotLocalUrl:
+            f = tempfile.SpooledTemporaryFile(prefix="tmp_pack_")
+            path = None
+        else:
+            f = tempfile.NamedTemporaryFile(dir=dir, prefix="tmp_pack_", delete=False)
+            path = f.name
+
+        def commit():
+            if f.tell() > 0:
+                f.seek(0)
+                with PackData(path, f) as pd:
+                    indexer = PackIndexer.for_pack_data(
+                        pd, resolve_ext_ref=self.get_raw
+                    )
+                    return self._complete_pack(f, path, len(pd), indexer)
+            else:
+                abort()
+                return None
+
+        def abort():
+            f.close()
+            if path:
+                os.remove(path)
+
+        return f, commit, abort
